uniform lowp float ColorScale;
uniform lowp float AlphaScale;

uniform lowp vec3 lightDiffuse[MAX_LIGHTS];
uniform lowp vec3 lightAmbient[MAX_LIGHTS];

#ifdef USE_FOG
uniform lowp vec4 fogColor;
#endif

#ifdef USE_SHADOWS
uniform highp vec2 shadowMapSize;
uniform lowp float lightMaxShadow[MAX_LIGHTS];
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
#endif

uniform sampler2D diffuseMap;

#ifdef USE_SHADOWS
uniform sampler2D shadowMap;
#endif

varying highp vec3 vNormal;
varying highp vec3 vLightDir;
varying highp vec2 vUv;

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef USE_SHADOWS
varying highp vec3 vPosShadowMapSpaceBiased;
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vPosWS;
#endif


const lowp float k_ambientIntensity = 0.3;


#ifdef USE_SHADOWS

#if defined( ADVANCED_SHADOWS )
#include "pcf_advanced.glsl"
#elif defined( HIGH_QUALITY_SHADOWS )
#include "pcf_high.glsl"
#elif defined( NORMAL_QUALITY_SHADOWS )
#include "pcf_normal.glsl"
#else
#include "pcf_low.glsl"
#endif

#endif


void main()
{
	lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUv );
	highp vec3 normal = normalize( vNormal );
	
	lowp float NdotL = dot( normal, vLightDir );
	lowp float diffuseIntensity = max( NdotL, 0.0 );
	diffuseIntensity *= ( 1.0 - k_ambientIntensity );
	
#ifdef CHARACTER_LIGHT
	lowp vec3 charLightDisp = charLightPos - vPosWS;
	lowp float charLightDist = length( charLightDisp );
	lowp vec3 charLightDir = charLightDisp / charLightDist;
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;
#endif
	
	lowp vec3 color = ( lightDiffuse[0] * diffuseCol.rgb * diffuseIntensity + diffuseCol.rgb * lightAmbient[0] * k_ambientIntensity ) * ColorScale;
	
#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor;
#endif

#ifdef USE_SHADOWS
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpaceBiased.x >= 0.0, vPosShadowMapSpaceBiased.x < 1.0, vPosShadowMapSpaceBiased.y >= 0.0, vPosShadowMapSpaceBiased.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpaceBiased.xy, vPosShadowMapSpaceBiased.z ) : 1.0;
	color *= mix(1.0, shadowValue, lightMaxShadow[0]);
#endif

#ifdef USE_FOG	
	color = mix( color, fogColor.rgb, vFogFactor );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	gl_FragColor = finalColor;
}
