uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientIntensity;

#ifdef USE_SPEC
uniform lowp float SpecularPower;
uniform lowp float SpecularIntensity;
uniform lowp vec3 SpecularColor;
#endif

#ifdef USE_FOG
uniform lowp vec4 fogColor;
#endif

#ifdef USE_SHADOWS
uniform highp vec2 shadowMapSize;
uniform lowp float ShadowTerm;
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
#endif

uniform sampler2D diffuseMap;

#ifdef USE_SHADOWS
uniform sampler2D shadowMap;
#endif

varying highp vec3 vNormal;
varying highp vec3 vLightDir;
varying highp vec2 vUv;

#ifdef USE_SPEC
varying highp vec3 vEyeDir;
#endif

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef USE_SHADOWS
varying highp vec3 vPosShadowMapSpaceBiased;
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vPosWS;
#endif

#if defined( CHARACTER_LIGHT ) && defined( USE_CHARACTER_LIGHT_SPEC )
const lowp float k_charSpecMaxRadius = 10.0;
#endif


#ifdef USE_SHADOWS

#if defined( ADVANCED_SHADOWS )
#include "pcf_advanced.glsl"
#elif defined( HIGH_QUALITY_SHADOWS )
#include "pcf_high.glsl"
#elif defined( NORMAL_QUALITY_SHADOWS )
#include "pcf_normal.glsl"
#else
#include "pcf_low.glsl"
#endif

#endif

#undef USE_SPEC

void main()
{
	lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUv );
	highp vec3 normal = normalize( vNormal );
	
	lowp float NdotL = dot( normal, vLightDir );
	lowp float diffuseIntensity = max( NdotL, AmbientIntensity );
	
#ifdef USE_SHADOWS
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpaceBiased.x >= 0.0, vPosShadowMapSpaceBiased.x < 1.0, vPosShadowMapSpaceBiased.y >= 0.0, vPosShadowMapSpaceBiased.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpaceBiased.xy, vPosShadowMapSpaceBiased.z ) : 1.0;
	diffuseIntensity = diffuseIntensity - ( 1.0 - shadowValue ) * ShadowTerm;
#endif
	
#ifdef CHARACTER_LIGHT
	lowp vec3 charLightDisp = charLightPos - vPosWS;
	lowp float charLightDist = length( charLightDisp );
	lowp vec3 charLightDir = charLightDisp / charLightDist;
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;

#endif

	lowp vec3 color = diffuseCol.rgb * diffuseIntensity * ColorScale;
	
#if defined( CHARACTER_LIGHT ) && defined( USE_CHARACTER_LIGHT_SPEC )
	lowp vec3 charLightSpecColor = vec3( 0.0, 0.0, 0.0 );
	if( charLightNdotL > 0.0 )
	{
		lowp vec3 reflectDir = reflect( -charLightDir, normal );
		lowp float factor = max( dot( normalize( vEyeDir ), reflectDir ), 0.0 );
		lowp float att = 1.0 - min( charLightDist / k_charSpecMaxRadius, 1.0 );
		charLightSpecColor = SpecularColor * SpecularIntensity * att * pow( factor, SpecularPower );
	}

	color += charLightSpecColor;
#endif
	
#ifdef USE_SPEC
	lowp vec3 specColor = vec3( 0.0, 0.0, 0.0 );
	if( NdotL > 0.0 )
	{
		lowp vec3 reflectDir = reflect( -vLightDir, normal );
		lowp float factor = max( dot( normalize( vEyeDir ), reflectDir ), 0.0 );
		specColor = SpecularColor * SpecularIntensity * pow( factor, SpecularPower );
	}
	
	color += specColor;
#endif
	
#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor;
#endif

#ifdef USE_FOG	
	color = mix( color, fogColor.rgb, vFogFactor );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	gl_FragColor = finalColor;
}
